/*
*  Licensed Materials - Property of IBM
*  
*  CICS SupportPac CA1T
* 
*   Copyright IBM Corporation 2008, 2010
*  All Rights Reserved
*  
*  US Government Users Restricted Rights  Use,
*  duplication, or disclosure restricted by GSA ADP
*  Schedule Contract with IBM Corporation
 */

/*
 *      File Name:    RequestValidationExit.java
*       Version:      1.0 
 *      Date:         271108
 *      Product:      CICS Transaction Gateway for z/OS
 * 
 * 
 * Description: The RequestValiadation exit is designed to validate the parameters in an ECI request
 * 
 * See SupportPac documentation for further details on how to configure and use
 * this sample
 */

package com.ibm.ctg.samples.ha;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Hashtable;
import java.util.HashSet;
import java.util.Set;

import java.util.Map;


import com.ibm.ctg.ha.CICSRequestExit;
import com.ibm.ctg.ha.RequestDetails;
import com.ibm.ctg.ha.InvalidRequestException;
import com.ibm.jzos.ZUtil;

public class RequestValidationExit implements CICSRequestExit {
	//                                             
	public static final String copyright = "\nLicensed Materials - Property of IBM "+
	"CICS SupportPac CA1T" +
	"(c) Copyright IBM Corp. 2008, 2010  All Rights Reserved. " +
	"US Government Users Restricted Rights - Use, duplication or " +
	"disclosure restricted by GSA ADP Schedule Contract with " +
	"IBM Corp.\n";
	//

	private static String jobname; // CTG job name
	//public static final String VERSION = "1.1";
    public static final String STDOUT_PROPERTY = "com.ibm.ctg.samples.cicsrequestexit.out"; // logging destination

    public static final String ENV_FILE = "CTG_HACONFIG"; // config file 

    public static final String TRANSID = "TRANSID"; // Keys in config file
    public static final String USERID = "USERID";
    public static final String PROGID = "PROGRAM";

    private String[] transidarray;
    private String[] progarray;
    private String[] useridarray;

    private Set<String> transidSet = new HashSet<String>(); // Set of tranids
    private Set<String> progSet   = new HashSet<String>(); // Set of progs
    private Set<String> useridSet = new HashSet<String>(); // Set of users
    private Hashtable<String, String[]> tables = new Hashtable<String, String[]>(); // Table

    HAUtil util = new HAUtil(); //util object

    /**
     * Constructor called at exit initialisation by Gateway 
     */
    public RequestValidationExit() throws Exception {
    	
    	jobname = ZUtil.getCurrentJobname(); // CTG jobname

        try {

            util.getstdout(); // Setup output destination
                        
            util.println("Starting CICS request exit " 
            		+ getClass().getSimpleName()
            		+ " V"+ HAUtil.VERSION
            		+ " in Gateway " + jobname);
            try {
                String configfile = ZUtil.getEnv(ENV_FILE);
                if (configfile == null) {
                    throw new IOException(ENV_FILE + " variable not set");
                }

                tables = util.readfile(util.openfile()); // Build the hash table from the config file

            } catch (FileNotFoundException fe) {
                util.println ("Error file not found " + fe.toString());
                throw new Exception (fe);
            } catch (IOException ioe) {
                util.println ("Error reading file " + ioe.toString());
                throw new Exception (ioe);
            }

            int i;
            int len;
            StringBuffer msg;

            // Build the array for the list of valid transactions
            if (tables.get(TRANSID) != null ) {
                transidarray = tables.get(TRANSID); 

                i = 0; 
                len = 0;
                while ( i < transidarray.length) {
                    transidSet.add(transidarray[i]);
                    i ++;
                }
                len = transidarray.length;
                msg = new StringBuffer();
                msg.append("Transaction ID selection built with transactions: ");
                i = 0;
                while (i < len) {
                    msg.append(transidarray[i] + ",");
                    i++;
                }
                util.println(msg.toString());
            }


            // Build the array for the list of valid programs
            if (tables.get(PROGID) != null ) {
                progarray = tables.get(PROGID); 

                i = 0; 
                while ( i < progarray.length) {
                    progSet.add(progarray[i]);
                    i ++;
                }       
                len = progarray.length;
                msg = new StringBuffer();
                msg.append("Program selection built with programs: ");
                i = 0;
                while (i < len) {
                    msg.append(progarray[i] + ",");
                    i++;
                }
                util.println(msg.toString());
            }



            // Build the array for the list of valid users
            if (tables.get(USERID) != null ) {
                useridarray = tables.get(USERID); 

                i = 0; 
                while ( i < useridarray.length) {
                    useridSet.add(useridarray[i]);
                    i ++;
                }
                len = useridarray.length;
                msg = new StringBuffer();
                msg.append("User ID selection built with users: ");
                i = 0;
                while (i < len) {
                    msg.append(useridarray[i] + ",");
                    i++;
                }
                util.println(msg.toString());
            }



        } catch (Exception e) {
            util.println ("Terminating with " + e.toString());
            throw new Exception (e);
        }

    }


    /**
     * getCICSServer is called for each ECI request to CICS to allow the request to be validated
     * and the server name to be amended
     * 
     *  Returns name of server 
     */
    public String getCICSServer(Map<RequestDetails, Object> data) throws InvalidRequestException {


        String program, transid, userid;
        String cicsServer = null;
        Object dataObject;

        cicsServer = (String) data.get(RequestDetails.Server); // Don't remap server name

        dataObject = data.get(RequestDetails.Transid);
        if (dataObject == null) {
            transid = "NULL";
        } else {
            transid = dataObject.toString();
        }


        // check if the request parameters are not specified in the set
        if ( !transidSet.contains(transid.toUpperCase()) && (transidarray!=null ) ) {
            util.traceln( getClass().getSimpleName() + " rejected transid:" +
                          transid + " to server:" + cicsServer ) ;
            throw new InvalidRequestException(HAUtil.id + " Rejected" + transid);

        }

        dataObject = data.get(RequestDetails.Program);
        if (dataObject == null) {
            program = "NULL";
        } else {
            program = dataObject.toString();
        }

        //check if the request parameters are not specified in the set
        if ( !progSet.contains(program.toUpperCase()) && (progarray!=null ) ) {
            util.traceln( getClass().getSimpleName() + " rejected program:" +
                          program + " to server:" + cicsServer)  ;
            throw new InvalidRequestException(HAUtil.id + " rejected" + program);   
        }

        dataObject = data.get(RequestDetails.Userid);
        if (dataObject == null) {
            userid = "NULL";
        } else {
            userid = dataObject.toString();
        }

        //check if the request parameters are not specified in the hash sets
        if ( !useridSet.contains(userid.toUpperCase()) && (useridarray!=null ) ) {
            util.traceln ( getClass().getSimpleName() + " rejected userid:" +
                           userid + " to server:" + cicsServer ) ;
            throw new InvalidRequestException(HAUtil.id + " rejected" + userid);    
        }


        // if OK then return input server as output

        util.traceln (" Validated request to server:" + cicsServer + 
                      " with transid:" + transid +
                      " userid:" + userid +
                      " program:" + program ) ;

        return cicsServer;

    }

    /**
     * Called at Gateway startup to set the max number of retries the Gateway will issue for an ECI
     * request if a request fails
     * 
     * Returns retry count 
     */
    public int getRetryCount() {

        return 0 ; //Don't retry requests in this exit
    }

    /**
     * Called at Shutdwon of the Gateway
     */
    public void shutdown() {

        util.println("Shutdown initiated in " + getClass().getSimpleName());
    }

    /**
     * 
     * Returns name of the current class
     */
    public String myName() {
        return this.getClass().getCanonicalName();
    }

}
