/*
*  Licensed Materials - Property of IBM
*  
*  CICS SupportPac CA1T
* 
*   Copyright IBM Corporation 2008, 2010
*  All Rights Reserved
*  
*  US Government Users Restricted Rights  Use,
*  duplication, or disclosure restricted by GSA ADP
*  Schedule Contract with IBM Corporation
 */

/*
 *      File Name:     RoundRobin.java
*       Version:      1.0 
 *      Date:         271108
 *      Product:      CICS Transaction Gateway for z/OS
 * 
 * 
 * Description: The RoundRobin exit is designed to redirect an ECI request across
 * a group of cloned CICS servers
 * 
 * See SupportPac documentation for further details on how to configure and use
 * this sample
 */


package com.ibm.ctg.samples.ha;

import java.util.Hashtable;
import java.util.Map;
import java.util.HashMap;

import com.ibm.ctg.ha.CICSRequestExit;
import com.ibm.ctg.ha.RequestDetails;

import com.ibm.jzos.ZUtil;

public class RoundRobinExit implements CICSRequestExit {
	//                                             
	public static final String copyright = "\nLicensed Materials - Property of IBM "+
	"CICS SupportPac CA1T" +
	"(c) Copyright IBM Corp. 2008, 2010  All Rights Reserved. " +
	"US Government Users Restricted Rights - Use, duplication or " +
	"disclosure restricted by GSA ADP Schedule Contract with " +
	"IBM Corp.\n";
	//


    public static final String ENV_FILE = "CTG_HACONFIG"; // config file 

    private static String jobname; // CTG job name

    private Hashtable<String, String[]> tables ; // Table of key/values from config file

    private HashMap<String, Integer> retryCache; // Table of thread + Server used

    private String[] serverArray; // server array for HA

    private int len; // Number of servers in HA serverArray	

    private int  nextServerIndex = 0; // Index in thread table

    private Object lock = new Object();

    HAUtil util = new HAUtil(); //util helper class

    /**
     * Constructor called at exit initialisation by Gateway 
     */
    public RoundRobinExit() throws Exception {


        try {

            jobname = ZUtil.getCurrentJobname(); // CTG jobname
            util.getstdout();                    // Setup output destination

            util.println("Starting CICS request exit " 
            		+ getClass().getSimpleName()
            		+ " V"+ HAUtil.VERSION
            		+ " in Gateway " + jobname);

            tables = util.readfile(util.openfile()); // Build the hash table from the config file

            serverArray = tables.get(jobname); // Build the server list for this CTG
            if (serverArray != null) {
                util.traceln("Tables built succesfully ");
                len = serverArray.length;
            } else {
                throw new Exception ("Jobname not found in configuration file");
            }


            // Thread table retryCache is used to store the history of retrys that have
            // been driven on this worker thread instance
            retryCache = new HashMap<String, Integer>();   

            StringBuffer msg = new StringBuffer();
            msg.append("Server selection for " + jobname + " built with servers: ");
            int i = 0;
            while (i < len) {
                msg.append(serverArray[i] + ",");
                i++;
            }
            util.println ( msg.toString());    

        } catch (Exception e) {
            util.println ("Terminating with " + e.toString());
            throw new Exception (e);
        }
    }


    /**
     * getCICSServer is called for each ECI request to CICS to allow the request to be validated
     * and the server name amended 
     */
    public String getCICSServer(Map<RequestDetails, Object> data) {


        String server = ""; // server for input
        String cicsserver = ""; // Server to be returned
        Object dataObject;
        String thread = Thread.currentThread().getName(); // ID of current worker thread
        int requestCount = (Integer)data.get(RequestDetails.RetryCount); // The retry count for this request 0=1st, 1=2nd



        dataObject = data.get(RequestDetails.Server);
        if (dataObject != null) {
            server = dataObject.toString(); // original server;
        }

        synchronized(lock){ // serialise this section of code to ensure we round robin through the table

            // if first time through initialise the retryCache table to store the server
            if (requestCount == 0) {
                cicsserver = serverArray[nextServerIndex];
                retryCache.put(thread, nextServerIndex);
                nextServerIndex = (nextServerIndex + 1) % serverArray.length;
            } else {

                // if request is being retried check the table to find last server used
                // and iterate through  serverArray to find a working server
                int serverIndex = retryCache.get(Thread.currentThread().getName());
                serverIndex = (serverIndex + 1) % serverArray.length;
                cicsserver = serverArray[serverIndex];
                retryCache.put(thread, serverIndex);   // update the table to store the server used

            }
        }
        util.traceln (" Retry:" + requestCount +
                      " thread: " + thread +
                      " remapped server:" + server  +
                      " to:" + cicsserver);

        return cicsserver;     
    }

    /**
     * Called at Gateway startup to set the max number of retries the Gateway will issue for an ECI
     * request if a request fails
     * 
     * Returns retry count 
     */
    public int getRetryCount() {

        //Set the number of times to retry request based on number of servers in array
        return serverArray.length-1;

    }

    /**
     * Returns name of the current class
     */
    public String myName() {
        return this.getClass().getCanonicalName();
    }


    /**
     * Called at Shutdwon of the Gateway
     */
    public void shutdown() {

        util.println("Shutdown initiated in " + getClass().getSimpleName());
    }



}

